#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: !0
});
const _path = _interopRequireDefault(require("path")), _arg = _interopRequireDefault(require("arg")), _fs = _interopRequireDefault(require("fs")), _build = require("./build"), _help = require("./help"), _init = require("./init");
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function oneOf(...options) {
    return Object.assign((value = !0)=>{
        for (let option of options){
            let parsed = option(value);
            if (parsed === value) return parsed;
        }
        throw Error("...");
    }, {
        manualParsing: !0
    });
}
let commands = {
    init: {
        run: _init.init,
        args: {
            "--esm": {
                type: Boolean,
                description: "Initialize configuration file as ESM"
            },
            "--ts": {
                type: Boolean,
                description: "Initialize configuration file as TypeScript"
            },
            "--postcss": {
                type: Boolean,
                description: "Initialize a `postcss.config.js` file"
            },
            "--full": {
                type: Boolean,
                description: "Include the default values for all options in the generated configuration file"
            },
            "-f": "--full",
            "-p": "--postcss"
        }
    },
    build: {
        run: _build.build,
        args: {
            "--input": {
                type: String,
                description: "Input file"
            },
            "--output": {
                type: String,
                description: "Output file"
            },
            "--watch": {
                type: oneOf(String, Boolean),
                description: "Watch for changes and rebuild as needed"
            },
            "--poll": {
                type: Boolean,
                description: "Use polling instead of filesystem events when watching"
            },
            "--content": {
                type: String,
                description: "Content paths to use for removing unused classes"
            },
            "--purge": {
                type: String,
                deprecated: !0
            },
            "--postcss": {
                type: oneOf(String, Boolean),
                description: "Load custom PostCSS configuration"
            },
            "--minify": {
                type: Boolean,
                description: "Minify the output"
            },
            "--config": {
                type: String,
                description: "Path to a custom config file"
            },
            "--no-autoprefixer": {
                type: Boolean,
                description: "Disable autoprefixer"
            },
            "-c": "--config",
            "-i": "--input",
            "-o": "--output",
            "-m": "--minify",
            "-w": "--watch",
            "-p": "--poll"
        }
    }
}, sharedFlags = {
    "--help": {
        type: Boolean,
        description: "Display usage information"
    },
    "-h": "--help"
};
process.stdout.isTTY && (void 0 === process.argv[2] || process.argv.slice(2).every((flag)=>void 0 !== sharedFlags[flag])) && ((0, _help.help)({
    usage: [
        "tailwindcss [--input input.css] [--output output.css] [--watch] [options...]",
        "tailwindcss init [--full] [--postcss] [options...]"
    ],
    commands: Object.keys(commands).filter((command)=>"build" !== command).map((command)=>`${command} [options]`),
    options: {
        ...commands.build.args,
        ...sharedFlags
    }
}), process.exit(0));
let command = ((arg = "")=>arg.startsWith("-") ? void 0 : arg)(process.argv[2]) || "build";
void 0 === commands[command] && (_fs.default.existsSync(_path.default.resolve(command)) ? command = "build" : ((0, _help.help)({
    message: `Invalid command: ${command}`,
    usage: [
        "tailwindcss <command> [options]"
    ],
    commands: Object.keys(commands).filter((command)=>"build" !== command).map((command)=>`${command} [options]`),
    options: sharedFlags
}), process.exit(1)));
let { args: flags , run  } = commands[command], args = (()=>{
    try {
        let result = (0, _arg.default)(Object.fromEntries(Object.entries({
            ...flags,
            ...sharedFlags
        }).filter(([_key, value])=>{
            var _value_type;
            return !(null == value ? void 0 : null === (_value_type = value.type) || void 0 === _value_type ? void 0 : _value_type.manualParsing);
        }).map(([key, value])=>[
                key,
                "object" == typeof value ? value.type : value
            ])), {
            permissive: !0
        });
        for(let i = result._.length - 1; i >= 0; --i){
            let flag = result._[i];
            if (!flag.startsWith("-")) continue;
            let [flagName, flagValue] = flag.split("="), handler = flags[flagName];
            for(; "string" == typeof handler;)flagName = handler, handler = flags[handler];
            if (!handler) continue;
            let args = [], offset = i + 1;
            if (void 0 === flagValue) {
                for(; result._[offset] && !result._[offset].startsWith("-");)args.push(result._[offset++]);
                result._.splice(i, 1 + args.length), flagValue = 0 === args.length ? void 0 : 1 === args.length ? args[0] : args;
            } else result._.splice(i, 1);
            result[flagName] = handler.type(flagValue, flagName);
        }
        return result._[0] !== command && result._.unshift(command), result;
    } catch (err) {
        throw "ARG_UNKNOWN_OPTION" === err.code && ((0, _help.help)({
            message: err.message,
            usage: [
                "tailwindcss <command> [options]"
            ],
            options: sharedFlags
        }), process.exit(1)), err;
    }
})();
args["--help"] && ((0, _help.help)({
    options: {
        ...flags,
        ...sharedFlags
    },
    usage: [
        `tailwindcss ${command} [options]`
    ]
}), process.exit(0)), run(args);
