"use strict";
Object.defineProperty(exports, "__esModule", {
    value: !0
}), Object.defineProperty(exports, "default", {
    enumerable: !0,
    get: ()=>expandTailwindAtRules
});
const _fs = _interopRequireDefault(require("fs")), _quickLru = _interopRequireDefault(require("quick-lru")), _sharedState = function(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) return obj;
    if (null === obj || "object" != typeof obj && "function" != typeof obj) return {
        default: obj
    };
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) return cache.get(obj);
    var newObj = {}, hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj)if ("default" !== key && Object.prototype.hasOwnProperty.call(obj, key)) {
        var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
        desc && (desc.get || desc.set) ? Object.defineProperty(newObj, key, desc) : newObj[key] = obj[key];
    }
    return newObj.default = obj, cache && cache.set(obj, newObj), newObj;
}(require("./sharedState")), _generateRules = require("./generateRules"), _log = _interopRequireDefault(require("../util/log")), _cloneNodes = _interopRequireDefault(require("../util/cloneNodes")), _defaultExtractor = require("./defaultExtractor");
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache(nodeInterop) {
    if ("function" != typeof WeakMap) return null;
    var cacheBabelInterop = new WeakMap(), cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
let env = _sharedState.env;
const builtInExtractors = {
    DEFAULT: _defaultExtractor.defaultExtractor
}, builtInTransformers = {
    DEFAULT: (content)=>content,
    svelte: (content)=>content.replace(/(?:^|\s)class:/g, " ")
};
let extractorCache = new WeakMap();
function expandTailwindAtRules(context) {
    return (root)=>{
        var _context_candidates;
        let layerNodes = {
            base: null,
            components: null,
            utilities: null,
            variants: null
        };
        if (root.walkAtRules((rule)=>{
            "tailwind" === rule.name && Object.keys(layerNodes).includes(rule.params) && (layerNodes[rule.params] = rule);
        }), Object.values(layerNodes).every((n)=>null === n)) return root;
        let candidates = new Set([
            ...null !== (_context_candidates = context.candidates) && void 0 !== _context_candidates ? _context_candidates : [],
            _sharedState.NOT_ON_DEMAND
        ]), seen = new Set();
        for (let { file , content , extension  } of (env.DEBUG && console.time("Reading changed files"), context.changedContent)){
            let transformer = function(tailwindConfig, fileExtension) {
                let transformers = tailwindConfig.content.transform;
                return transformers[fileExtension] || transformers.DEFAULT || builtInTransformers[fileExtension] || builtInTransformers.DEFAULT;
            }(context.tailwindConfig, extension), extractor = function(context, fileExtension) {
                let extractors = context.tailwindConfig.content.extract;
                return extractors[fileExtension] || extractors.DEFAULT || builtInExtractors[fileExtension] || builtInExtractors.DEFAULT(context);
            }(context, extension);
            !function(content, extractor, candidates, seen) {
                for (let line of (extractorCache.has(extractor) || extractorCache.set(extractor, new _quickLru.default({
                    maxSize: 25000
                })), content.split("\n")))if (line = line.trim(), !seen.has(line)) {
                    if (seen.add(line), extractorCache.get(extractor).has(line)) for (let match of extractorCache.get(extractor).get(line))candidates.add(match);
                    else {
                        let extractorMatches = extractor(line).filter((s)=>"!*" !== s), lineMatchesSet = new Set(extractorMatches);
                        for (let match1 of lineMatchesSet)candidates.add(match1);
                        extractorCache.get(extractor).set(line, lineMatchesSet);
                    }
                }
            }(transformer(content = file ? _fs.default.readFileSync(file, "utf8") : content), extractor, candidates, seen);
        }
        env.DEBUG && console.timeEnd("Reading changed files");
        let classCacheCount = context.classCache.size;
        env.DEBUG && console.time("Generate rules"), env.DEBUG && console.time("Sorting candidates");
        let sortedCandidates = new Set([
            ...candidates
        ].sort((a, z)=>a === z ? 0 : a < z ? -1 : 1));
        env.DEBUG && console.timeEnd("Sorting candidates"), (0, _generateRules.generateRules)(sortedCandidates, context), env.DEBUG && console.timeEnd("Generate rules"), env.DEBUG && console.time("Build stylesheet"), (null === context.stylesheetCache || context.classCache.size !== classCacheCount) && (context.stylesheetCache = function(rules, context) {
            let sortedRules = context.offsets.sort(rules), returnValue = {
                base: new Set(),
                defaults: new Set(),
                components: new Set(),
                utilities: new Set(),
                variants: new Set()
            };
            for (let [sort, rule] of sortedRules)returnValue[sort.layer].add(rule);
            return returnValue;
        }([
            ...context.ruleCache
        ], context)), env.DEBUG && console.timeEnd("Build stylesheet");
        let { defaults: defaultNodes , base: baseNodes , components: componentNodes , utilities: utilityNodes , variants: screenNodes  } = context.stylesheetCache;
        layerNodes.base && (layerNodes.base.before((0, _cloneNodes.default)([
            ...baseNodes,
            ...defaultNodes
        ], layerNodes.base.source, {
            layer: "base"
        })), layerNodes.base.remove()), layerNodes.components && (layerNodes.components.before((0, _cloneNodes.default)([
            ...componentNodes
        ], layerNodes.components.source, {
            layer: "components"
        })), layerNodes.components.remove()), layerNodes.utilities && (layerNodes.utilities.before((0, _cloneNodes.default)([
            ...utilityNodes
        ], layerNodes.utilities.source, {
            layer: "utilities"
        })), layerNodes.utilities.remove());
        let variantNodes = Array.from(screenNodes).filter((node)=>{
            var _node_raws_tailwind;
            let parentLayer = null === (_node_raws_tailwind = node.raws.tailwind) || void 0 === _node_raws_tailwind ? void 0 : _node_raws_tailwind.parentLayer;
            return "components" === parentLayer ? null !== layerNodes.components : "utilities" !== parentLayer || null !== layerNodes.utilities;
        });
        layerNodes.variants ? (layerNodes.variants.before((0, _cloneNodes.default)(variantNodes, layerNodes.variants.source, {
            layer: "variants"
        })), layerNodes.variants.remove()) : variantNodes.length > 0 && root.append((0, _cloneNodes.default)(variantNodes, root.source, {
            layer: "variants"
        }));
        let hasUtilityVariants = variantNodes.some((node)=>{
            var _node_raws_tailwind;
            return (null === (_node_raws_tailwind = node.raws.tailwind) || void 0 === _node_raws_tailwind ? void 0 : _node_raws_tailwind.parentLayer) === "utilities";
        });
        layerNodes.utilities && 0 === utilityNodes.size && !hasUtilityVariants && _log.default.warn("content-problems", [
            "No utility classes were detected in your source files. If this is unexpected, double-check the `content` option in your Tailwind CSS configuration.",
            "https://tailwindcss.com/docs/content-configuration"
        ]), env.DEBUG && (console.log("Potential classes: ", candidates.size), console.log("Active contexts: ", _sharedState.contextSourcesMap.size)), context.changedContent = [], root.walkAtRules("layer", (rule)=>{
            Object.keys(layerNodes).includes(rule.params) && rule.remove();
        });
    };
}
