"use strict";
Object.defineProperty(exports, "__esModule", {
    value: !0
}), Object.defineProperty(exports, "Offsets", {
    enumerable: !0,
    get: ()=>Offsets
});
const _bigSign = function(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}(require("../util/bigSign")), _remapBitfieldJs = require("./remap-bitfield.js");
class Offsets {
    constructor(){
        this.offsets = {
            defaults: 0n,
            base: 0n,
            components: 0n,
            utilities: 0n,
            variants: 0n,
            user: 0n
        }, this.layerPositions = {
            defaults: 0n,
            base: 1n,
            components: 2n,
            utilities: 3n,
            user: 4n,
            variants: 5n
        }, this.reservedVariantBits = 0n, this.variantOffsets = new Map();
    }
    create(layer) {
        return {
            layer,
            parentLayer: layer,
            arbitrary: 0n,
            variants: 0n,
            parallelIndex: 0n,
            index: this.offsets[layer]++,
            options: []
        };
    }
    arbitraryProperty() {
        return {
            ...this.create("utilities"),
            arbitrary: 1n
        };
    }
    forVariant(variant, index = 0) {
        let offset = this.variantOffsets.get(variant);
        if (void 0 === offset) throw Error(`Cannot find offset for unknown variant ${variant}`);
        return {
            ...this.create("variants"),
            variants: offset << BigInt(index)
        };
    }
    applyVariantOffset(rule, variant, options) {
        return options.variant = variant.variants, {
            ...rule,
            layer: "variants",
            parentLayer: "variants" === rule.layer ? rule.parentLayer : rule.layer,
            variants: rule.variants | variant.variants,
            options: options.sort ? [].concat(options, rule.options) : rule.options,
            parallelIndex: max([
                rule.parallelIndex,
                variant.parallelIndex
            ])
        };
    }
    applyParallelOffset(offset, parallelIndex) {
        return {
            ...offset,
            parallelIndex: BigInt(parallelIndex)
        };
    }
    recordVariants(variants, getLength) {
        for (let variant of variants)this.recordVariant(variant, getLength(variant));
    }
    recordVariant(variant, fnCount = 1) {
        return this.variantOffsets.set(variant, 1n << this.reservedVariantBits), this.reservedVariantBits += BigInt(fnCount), {
            ...this.create("variants"),
            variants: this.variantOffsets.get(variant)
        };
    }
    compare(a, b) {
        if (a.layer !== b.layer) return this.layerPositions[a.layer] - this.layerPositions[b.layer];
        if (a.parentLayer !== b.parentLayer) return this.layerPositions[a.parentLayer] - this.layerPositions[b.parentLayer];
        for (let aOptions of a.options)for (let bOptions of b.options){
            var _max;
            if (aOptions.id !== bOptions.id || !aOptions.sort || !bOptions.sort) continue;
            let maxFnVariant = null !== (_max = max([
                aOptions.variant,
                bOptions.variant
            ])) && void 0 !== _max ? _max : 0n, mask = ~(maxFnVariant | maxFnVariant - 1n);
            if ((a.variants & mask) != (b.variants & mask)) continue;
            let result = aOptions.sort({
                value: aOptions.value,
                modifier: aOptions.modifier
            }, {
                value: bOptions.value,
                modifier: bOptions.modifier
            });
            if (0 !== result) return result;
        }
        return a.variants !== b.variants ? a.variants - b.variants : a.parallelIndex !== b.parallelIndex ? a.parallelIndex - b.parallelIndex : a.arbitrary !== b.arbitrary ? a.arbitrary - b.arbitrary : a.index - b.index;
    }
    recalculateVariantOffsets() {
        let variants = Array.from(this.variantOffsets.entries()).filter(([v])=>v.startsWith("[")).sort(([a], [z])=>(function(a, b) {
                let aLen = a.length, bLen = b.length, minLen = aLen < bLen ? aLen : bLen;
                for(let i = 0; i < minLen; i++){
                    let cmp = a.charCodeAt(i) - b.charCodeAt(i);
                    if (0 !== cmp) return cmp;
                }
                return aLen - bLen;
            })(a, z)), newOffsets = variants.map(([, offset])=>offset).sort((a, z)=>(0, _bigSign.default)(a - z));
        return variants.map(([, oldOffset], i)=>[
                oldOffset,
                newOffsets[i]
            ]).filter(([a, z])=>a !== z);
    }
    remapArbitraryVariantOffsets(list) {
        let mapping = this.recalculateVariantOffsets();
        return 0 === mapping.length ? list : list.map((item)=>{
            let [offset, rule] = item;
            return [
                offset = {
                    ...offset,
                    variants: (0, _remapBitfieldJs.remapBitfield)(offset.variants, mapping)
                },
                rule
            ];
        });
    }
    sort(list) {
        return (list = this.remapArbitraryVariantOffsets(list)).sort(([a], [b])=>(0, _bigSign.default)(this.compare(a, b)));
    }
}
function max(nums) {
    let max = null;
    for (let num of nums)max = (max = null != max ? max : num) > num ? max : num;
    return max;
}
