"use strict";
Object.defineProperty(exports, "__esModule", {
    value: !0
}), function(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: !0,
        get: all[name]
    });
}(exports, {
    isValidVariantFormatString: ()=>isValidVariantFormatString,
    parseVariant: ()=>parseVariant,
    getFileModifiedMap: ()=>getFileModifiedMap,
    createContext: ()=>createContext,
    getContext: ()=>getContext
});
const _fs = _interopRequireDefault(require("fs")), _url = _interopRequireDefault(require("url")), _postcss = _interopRequireDefault(require("postcss")), _dlv = _interopRequireDefault(require("dlv")), _postcssSelectorParser = _interopRequireDefault(require("postcss-selector-parser")), _transformThemeValue = _interopRequireDefault(require("../util/transformThemeValue")), _parseObjectStyles = _interopRequireDefault(require("../util/parseObjectStyles")), _prefixSelector = _interopRequireDefault(require("../util/prefixSelector")), _isPlainObject = _interopRequireDefault(require("../util/isPlainObject")), _escapeClassName = _interopRequireDefault(require("../util/escapeClassName")), _nameClass = _interopRequireWildcard(require("../util/nameClass")), _pluginUtils = require("../util/pluginUtils"), _corePlugins = require("../corePlugins"), _sharedState = _interopRequireWildcard(require("./sharedState")), _toPath = require("../util/toPath"), _log = _interopRequireDefault(require("../util/log")), _negateValue = _interopRequireDefault(require("../util/negateValue")), _isSyntacticallyValidPropertyValue = _interopRequireDefault(require("../util/isSyntacticallyValidPropertyValue")), _generateRules = require("./generateRules"), _cacheInvalidationJs = require("./cacheInvalidation.js"), _offsetsJs = require("./offsets.js"), _featureFlagsJs = require("../featureFlags.js"), _formatVariantSelector = require("../util/formatVariantSelector");
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function _getRequireWildcardCache(nodeInterop) {
    if ("function" != typeof WeakMap) return null;
    var cacheBabelInterop = new WeakMap(), cacheNodeInterop = new WeakMap();
    return (_getRequireWildcardCache = function(nodeInterop) {
        return nodeInterop ? cacheNodeInterop : cacheBabelInterop;
    })(nodeInterop);
}
function _interopRequireWildcard(obj, nodeInterop) {
    if (!nodeInterop && obj && obj.__esModule) return obj;
    if (null === obj || "object" != typeof obj && "function" != typeof obj) return {
        default: obj
    };
    var cache = _getRequireWildcardCache(nodeInterop);
    if (cache && cache.has(obj)) return cache.get(obj);
    var newObj = {}, hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor;
    for(var key in obj)if ("default" !== key && Object.prototype.hasOwnProperty.call(obj, key)) {
        var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null;
        desc && (desc.get || desc.set) ? Object.defineProperty(newObj, key, desc) : newObj[key] = obj[key];
    }
    return newObj.default = obj, cache && cache.set(obj, newObj), newObj;
}
const VARIANT_TYPES = {
    AddVariant: Symbol.for("ADD_VARIANT"),
    MatchVariant: Symbol.for("MATCH_VARIANT")
}, VARIANT_INFO = {
    Base: 1,
    Dynamic: 2
};
function prefix(context, selector) {
    let prefix = context.tailwindConfig.prefix;
    return "function" == typeof prefix ? prefix(selector) : prefix + selector;
}
function normalizeOptionTypes({ type ="any" , ...options }) {
    let types = [].concat(type);
    return {
        ...options,
        types: types.map((type)=>Array.isArray(type) ? {
                type: type[0],
                ...type[1]
            } : {
                type,
                preferOnConflict: !1
            })
    };
}
function getClasses(selector, mutate) {
    return (0, _postcssSelectorParser.default)((selectors)=>{
        let allClasses = [];
        return mutate && mutate(selectors), selectors.walkClasses((classNode)=>{
            allClasses.push(classNode.value);
        }), allClasses;
    }).transformSync(selector);
}
function withIdentifiers(styles) {
    return (function parseStyles(styles) {
        return Array.isArray(styles) ? styles.flatMap((style)=>Array.isArray(style) || (0, _isPlainObject.default)(style) ? (0, _parseObjectStyles.default)(style) : style) : parseStyles([
            styles
        ]);
    })(styles).flatMap((node)=>{
        let nodeMap = new Map(), [containsNonOnDemandableSelectors, candidates] = function(node, state = {
            containsNonOnDemandable: !1
        }, depth = 0) {
            let classes = [];
            if ("rule" === node.type) {
                function ignoreNot(selectors) {
                    selectors.walkPseudos((pseudo)=>{
                        ":not" === pseudo.value && pseudo.remove();
                    });
                }
                for (let selector of node.selectors){
                    let classCandidates = getClasses(selector, ignoreNot);
                    for (let classCandidate of (0 === classCandidates.length && (state.containsNonOnDemandable = !0), classCandidates))classes.push(classCandidate);
                }
            } else "atrule" === node.type && node.walkRules((rule)=>{
                for (let classCandidate of rule.selectors.flatMap((selector)=>getClasses(selector)))classes.push(classCandidate);
            });
            return 0 === depth ? [
                state.containsNonOnDemandable || 0 === classes.length,
                classes
            ] : classes;
        }(node);
        return containsNonOnDemandableSelectors && candidates.unshift(_sharedState.NOT_ON_DEMAND), candidates.map((c)=>(nodeMap.has(node) || nodeMap.set(node, node), [
                c,
                nodeMap.get(node)
            ]));
    });
}
function isValidVariantFormatString(format) {
    return format.startsWith("@") || format.includes("&");
}
function parseVariant(variant) {
    let fns = (function(input) {
        let parts = [], current = "", depth = 0;
        for(let idx = 0; idx < input.length; idx++){
            let char = input[idx];
            if ("\\" === char) current += "\\" + input[++idx];
            else if ("{" === char) ++depth, parts.push(current.trim()), current = "";
            else if ("}" === char) {
                if (--depth < 0) throw Error("Your { and } are unbalanced.");
                parts.push(current.trim()), current = "";
            } else current += char;
        }
        return current.length > 0 && parts.push(current.trim()), parts = parts.filter((part)=>"" !== part);
    })(variant = variant.replace(/\n+/g, "").replace(/\s{1,}/g, " ").trim()).map((str)=>{
        if (!str.startsWith("@")) return ({ format  })=>format(str);
        let [, name, params] = /@(.*?)( .+|[({].*)/g.exec(str);
        return ({ wrap  })=>wrap(_postcss.default.atRule({
                name,
                params: params.trim()
            }));
    }).reverse();
    return (api)=>{
        for (let fn of fns)fn(api);
    };
}
let fileModifiedMapCache = new WeakMap();
function getFileModifiedMap(context) {
    return fileModifiedMapCache.has(context) || fileModifiedMapCache.set(context, new Map()), fileModifiedMapCache.get(context);
}
function trackModified(files, fileModifiedMap) {
    let changed = !1, mtimesToCommit = new Map();
    for (let file of files){
        var _fs_statSync;
        if (!file) continue;
        let parsed = _url.default.parse(file), pathname = parsed.hash ? parsed.href.replace(parsed.hash, "") : parsed.href;
        pathname = parsed.search ? pathname.replace(parsed.search, "") : pathname;
        let newModified = null === (_fs_statSync = _fs.default.statSync(decodeURIComponent(pathname), {
            throwIfNoEntry: !1
        })) || void 0 === _fs_statSync ? void 0 : _fs_statSync.mtimeMs;
        newModified && ((!fileModifiedMap.has(file) || newModified > fileModifiedMap.get(file)) && (changed = !0), mtimesToCommit.set(file, newModified));
    }
    return [
        changed,
        mtimesToCommit
    ];
}
function markInvalidUtilityCandidate(context, candidate) {
    context.classCache.has(candidate) && (context.notClassCache.add(candidate), context.classCache.delete(candidate), context.applyClassCache.delete(candidate), context.candidateRuleMap.delete(candidate), context.candidateRuleCache.delete(candidate), context.stylesheetCache = null);
}
function createContext(tailwindConfig, changedContent = [], root = _postcss.default.root()) {
    var _tailwindConfig_blocklist;
    let corePluginList, userPlugins, layerPlugins;
    let context = {
        disposables: [],
        ruleCache: new Set(),
        candidateRuleCache: new Map(),
        classCache: new Map(),
        applyClassCache: new Map(),
        notClassCache: new Set(null !== (_tailwindConfig_blocklist = tailwindConfig.blocklist) && void 0 !== _tailwindConfig_blocklist ? _tailwindConfig_blocklist : []),
        postCssNodeCache: new Map(),
        candidateRuleMap: new Map(),
        tailwindConfig,
        changedContent: changedContent,
        variantMap: new Map(),
        stylesheetCache: null,
        variantOptions: new Map(),
        markInvalidUtilityCandidate: (candidate)=>markInvalidUtilityCandidate(context, candidate),
        markInvalidUtilityNode: (node)=>(function(context, node) {
                let candidate = node.raws.tailwind.candidate;
                if (candidate) {
                    for (let entry of context.ruleCache)entry[1].raws.tailwind.candidate === candidate && context.ruleCache.delete(entry);
                    markInvalidUtilityCandidate(context, candidate);
                }
            })(context, node)
    };
    return !function(plugins, context) {
        var _context_tailwindConfig_safelist, _context_tailwindConfig_darkMode, _concat_;
        let variantList = [], variantMap = new Map();
        context.variantMap = variantMap;
        let offsets = new _offsetsJs.Offsets();
        context.offsets = offsets;
        let classList = new Set(), pluginApi = function(tailwindConfig, context, { variantList , variantMap , offsets , classList  }) {
            function getConfigValue(path, defaultValue) {
                return path ? (0, _dlv.default)(tailwindConfig, path, defaultValue) : tailwindConfig;
            }
            function prefixIdentifier(identifier, options) {
                return identifier === _sharedState.NOT_ON_DEMAND ? _sharedState.NOT_ON_DEMAND : options.respectPrefix ? context.tailwindConfig.prefix + identifier : identifier;
            }
            let variantIdentifier = 0, api = {
                postcss: _postcss.default,
                prefix: function(selector) {
                    return (0, _prefixSelector.default)(tailwindConfig.prefix, selector);
                },
                e: _escapeClassName.default,
                config: getConfigValue,
                theme: function(path, defaultValue, opts = {}) {
                    let parts = (0, _toPath.toPath)(path), value = getConfigValue([
                        "theme",
                        ...parts
                    ], defaultValue);
                    return (0, _transformThemeValue.default)(parts[0])(value, opts);
                },
                corePlugins: (path)=>Array.isArray(tailwindConfig.corePlugins) ? tailwindConfig.corePlugins.includes(path) : getConfigValue([
                        "corePlugins",
                        path
                    ], !0),
                variants: ()=>[],
                addBase (base) {
                    for (let [identifier, rule] of withIdentifiers(base)){
                        let prefixedIdentifier = prefixIdentifier(identifier, {}), offset = offsets.create("base");
                        context.candidateRuleMap.has(prefixedIdentifier) || context.candidateRuleMap.set(prefixedIdentifier, []), context.candidateRuleMap.get(prefixedIdentifier).push([
                            {
                                sort: offset,
                                layer: "base"
                            },
                            rule
                        ]);
                    }
                },
                addDefaults (group, declarations) {
                    let groups = {
                        [`@defaults ${group}`]: declarations
                    };
                    for (let [identifier, rule] of withIdentifiers(groups)){
                        let prefixedIdentifier = prefixIdentifier(identifier, {});
                        context.candidateRuleMap.has(prefixedIdentifier) || context.candidateRuleMap.set(prefixedIdentifier, []), context.candidateRuleMap.get(prefixedIdentifier).push([
                            {
                                sort: offsets.create("defaults"),
                                layer: "defaults"
                            },
                            rule
                        ]);
                    }
                },
                addComponents (components, options) {
                    for (let [identifier, rule] of (options = Object.assign({}, {
                        preserveSource: !1,
                        respectPrefix: !0,
                        respectImportant: !1
                    }, Array.isArray(options) ? {} : options), withIdentifiers(components))){
                        let prefixedIdentifier = prefixIdentifier(identifier, options);
                        classList.add(prefixedIdentifier), context.candidateRuleMap.has(prefixedIdentifier) || context.candidateRuleMap.set(prefixedIdentifier, []), context.candidateRuleMap.get(prefixedIdentifier).push([
                            {
                                sort: offsets.create("components"),
                                layer: "components",
                                options
                            },
                            rule
                        ]);
                    }
                },
                addUtilities (utilities, options) {
                    for (let [identifier, rule] of (options = Object.assign({}, {
                        preserveSource: !1,
                        respectPrefix: !0,
                        respectImportant: !0
                    }, Array.isArray(options) ? {} : options), withIdentifiers(utilities))){
                        let prefixedIdentifier = prefixIdentifier(identifier, options);
                        classList.add(prefixedIdentifier), context.candidateRuleMap.has(prefixedIdentifier) || context.candidateRuleMap.set(prefixedIdentifier, []), context.candidateRuleMap.get(prefixedIdentifier).push([
                            {
                                sort: offsets.create("utilities"),
                                layer: "utilities",
                                options
                            },
                            rule
                        ]);
                    }
                },
                matchUtilities: function(utilities, options) {
                    options = normalizeOptionTypes({
                        respectPrefix: !0,
                        respectImportant: !0,
                        modifiers: !1,
                        ...options
                    });
                    let offset = offsets.create("utilities");
                    for(let identifier in utilities){
                        let prefixedIdentifier = prefixIdentifier(identifier, options), rule = utilities[identifier];
                        function wrapped(modifier, { isOnlyPlugin  }) {
                            let [value, coercedType, utilityModifier] = (0, _pluginUtils.coerceValue)(options.types, modifier, options, tailwindConfig);
                            if (void 0 === value) return [];
                            if (!options.types.some(({ type  })=>type === coercedType)) {
                                if (!isOnlyPlugin) return [];
                                _log.default.warn([
                                    `Unnecessary typehint \`${coercedType}\` in \`${identifier}-${modifier}\`.`,
                                    `You can safely update it to \`${identifier}-${modifier.replace(coercedType + ":", "")}\`.`
                                ]);
                            }
                            if (!(0, _isSyntacticallyValidPropertyValue.default)(value)) return [];
                            let extras = {
                                get modifier () {
                                    return options.modifiers || _log.default.warn(`modifier-used-without-options-for-${identifier}`, [
                                        "Your plugin must set `modifiers: true` in its options to support modifiers."
                                    ]), utilityModifier;
                                }
                            };
                            return [].concat((0, _featureFlagsJs.flagEnabled)(tailwindConfig, "generalizedModifiers") ? rule(value, extras) : rule(value)).filter(Boolean).map((declaration)=>({
                                    [(0, _nameClass.default)(identifier, modifier)]: declaration
                                }));
                        }
                        classList.add([
                            prefixedIdentifier,
                            options
                        ]);
                        let withOffsets = [
                            {
                                sort: offset,
                                layer: "utilities",
                                options
                            },
                            wrapped
                        ];
                        context.candidateRuleMap.has(prefixedIdentifier) || context.candidateRuleMap.set(prefixedIdentifier, []), context.candidateRuleMap.get(prefixedIdentifier).push(withOffsets);
                    }
                },
                matchComponents: function(components, options) {
                    options = normalizeOptionTypes({
                        respectPrefix: !0,
                        respectImportant: !1,
                        modifiers: !1,
                        ...options
                    });
                    let offset = offsets.create("components");
                    for(let identifier in components){
                        let prefixedIdentifier = prefixIdentifier(identifier, options), rule = components[identifier];
                        function wrapped(modifier, { isOnlyPlugin  }) {
                            let [value, coercedType, utilityModifier] = (0, _pluginUtils.coerceValue)(options.types, modifier, options, tailwindConfig);
                            if (void 0 === value) return [];
                            if (!options.types.some(({ type  })=>type === coercedType)) {
                                if (!isOnlyPlugin) return [];
                                _log.default.warn([
                                    `Unnecessary typehint \`${coercedType}\` in \`${identifier}-${modifier}\`.`,
                                    `You can safely update it to \`${identifier}-${modifier.replace(coercedType + ":", "")}\`.`
                                ]);
                            }
                            if (!(0, _isSyntacticallyValidPropertyValue.default)(value)) return [];
                            let extras = {
                                get modifier () {
                                    return options.modifiers || _log.default.warn(`modifier-used-without-options-for-${identifier}`, [
                                        "Your plugin must set `modifiers: true` in its options to support modifiers."
                                    ]), utilityModifier;
                                }
                            };
                            return [].concat((0, _featureFlagsJs.flagEnabled)(tailwindConfig, "generalizedModifiers") ? rule(value, extras) : rule(value)).filter(Boolean).map((declaration)=>({
                                    [(0, _nameClass.default)(identifier, modifier)]: declaration
                                }));
                        }
                        classList.add([
                            prefixedIdentifier,
                            options
                        ]);
                        let withOffsets = [
                            {
                                sort: offset,
                                layer: "components",
                                options
                            },
                            wrapped
                        ];
                        context.candidateRuleMap.has(prefixedIdentifier) || context.candidateRuleMap.set(prefixedIdentifier, []), context.candidateRuleMap.get(prefixedIdentifier).push(withOffsets);
                    }
                },
                addVariant (variantName, variantFunctions, options = {}) {
                    variantFunctions = [].concat(variantFunctions).map((variantFunction)=>{
                        if ("string" != typeof variantFunction) return (api = {})=>{
                            let { args , modifySelectors , container , separator , wrap , format  } = api, result = variantFunction(Object.assign({
                                modifySelectors,
                                container,
                                separator
                            }, options.type === VARIANT_TYPES.MatchVariant && {
                                args,
                                wrap,
                                format
                            }));
                            if ("string" == typeof result && !isValidVariantFormatString(result)) throw Error(`Your custom variant \`${variantName}\` has an invalid format string. Make sure it's an at-rule or contains a \`&\` placeholder.`);
                            return Array.isArray(result) ? result.filter((variant)=>"string" == typeof variant).map((variant)=>parseVariant(variant)) : result && "string" == typeof result && parseVariant(result)(api);
                        };
                        if (!isValidVariantFormatString(variantFunction)) throw Error(`Your custom variant \`${variantName}\` has an invalid format string. Make sure it's an at-rule or contains a \`&\` placeholder.`);
                        return parseVariant(variantFunction);
                    }), function(list, value, { before =[]  } = {}) {
                        if ((before = [].concat(before)).length <= 0) {
                            list.push(value);
                            return;
                        }
                        let idx = list.length - 1;
                        for (let other of before){
                            let iidx = list.indexOf(other);
                            -1 !== iidx && (idx = Math.min(idx, iidx));
                        }
                        list.splice(idx, 0, value);
                    }(variantList, variantName, options), variantMap.set(variantName, variantFunctions), context.variantOptions.set(variantName, options);
                },
                matchVariant (variant, variantFn, options) {
                    var _options_id, _options_values, _options_values1;
                    let id = null !== (_options_id = null == options ? void 0 : options.id) && void 0 !== _options_id ? _options_id : ++variantIdentifier, isSpecial = "@" === variant, modifiersEnabled = (0, _featureFlagsJs.flagEnabled)(tailwindConfig, "generalizedModifiers");
                    for (let [key, value] of Object.entries(null !== (_options_values = null == options ? void 0 : options.values) && void 0 !== _options_values ? _options_values : {}))"DEFAULT" !== key && api.addVariant(isSpecial ? `${variant}${key}` : `${variant}-${key}`, ({ args , container  })=>variantFn(value, modifiersEnabled ? {
                            modifier: null == args ? void 0 : args.modifier,
                            container
                        } : {
                            container
                        }), {
                        ...options,
                        value,
                        id,
                        type: VARIANT_TYPES.MatchVariant,
                        variantInfo: VARIANT_INFO.Base
                    });
                    let hasDefault = "DEFAULT" in (null !== (_options_values1 = null == options ? void 0 : options.values) && void 0 !== _options_values1 ? _options_values1 : {});
                    api.addVariant(variant, ({ args , container  })=>{
                        var _args_value;
                        return (null == args ? void 0 : args.value) !== _sharedState.NONE || hasDefault ? variantFn((null == args ? void 0 : args.value) === _sharedState.NONE ? options.values.DEFAULT : null !== (_args_value = null == args ? void 0 : args.value) && void 0 !== _args_value ? _args_value : "string" == typeof args ? args : "", modifiersEnabled ? {
                            modifier: null == args ? void 0 : args.modifier,
                            container
                        } : {
                            container
                        }) : null;
                    }, {
                        ...options,
                        id,
                        type: VARIANT_TYPES.MatchVariant,
                        variantInfo: VARIANT_INFO.Dynamic
                    });
                }
            };
            return api;
        }(context.tailwindConfig, context, {
            variantList,
            variantMap,
            offsets,
            classList
        });
        for (let plugin of plugins)if (Array.isArray(plugin)) for (let pluginItem of plugin)pluginItem(pluginApi);
        else null == plugin || plugin(pluginApi);
        for (let [variantName, variantFunctions] of (offsets.recordVariants(variantList, (variant)=>variantMap.get(variant).length), variantMap.entries()))context.variantMap.set(variantName, variantFunctions.map((variantFunction, idx)=>[
                offsets.forVariant(variantName, idx),
                variantFunction
            ]));
        let safelist = (null !== (_context_tailwindConfig_safelist = context.tailwindConfig.safelist) && void 0 !== _context_tailwindConfig_safelist ? _context_tailwindConfig_safelist : []).filter(Boolean);
        if (safelist.length > 0) {
            let checks = [];
            for (let value of safelist){
                if ("string" == typeof value) {
                    context.changedContent.push({
                        content: value,
                        extension: "html"
                    });
                    continue;
                }
                if (value instanceof RegExp) {
                    _log.default.warn("root-regex", [
                        "Regular expressions in `safelist` work differently in Tailwind CSS v3.0.",
                        "Update your `safelist` configuration to eliminate this warning.",
                        "https://tailwindcss.com/docs/content-configuration#safelisting-classes"
                    ]);
                    continue;
                }
                checks.push(value);
            }
            if (checks.length > 0) {
                let patternMatchingCount = new Map(), prefixLength = context.tailwindConfig.prefix.length, checkImportantUtils = checks.some((check)=>check.pattern.source.includes("!"));
                for (let util of classList)for (let util1 of Array.isArray(util) ? (()=>{
                    var _options_values;
                    let [utilName, options] = util, classes = Object.keys(null !== (_options_values = null == options ? void 0 : options.values) && void 0 !== _options_values ? _options_values : {}).map((value)=>(0, _nameClass.formatClass)(utilName, value));
                    return (null == options ? void 0 : options.supportsNegativeValues) && (classes = [
                        ...classes = [
                            ...classes,
                            ...classes.map((cls)=>"-" + cls)
                        ],
                        ...classes.map((cls)=>cls.slice(0, prefixLength) + "-" + cls.slice(prefixLength))
                    ]), options.types.some(({ type  })=>"color" === type) && (classes = [
                        ...classes,
                        ...classes.flatMap((cls)=>Object.keys(context.tailwindConfig.theme.opacity).map((opacity)=>`${cls}/${opacity}`))
                    ]), checkImportantUtils && (null == options ? void 0 : options.respectImportant) && (classes = [
                        ...classes,
                        ...classes.map((cls)=>"!" + cls)
                    ]), classes;
                })() : [
                    util
                ])for (let { pattern , variants =[]  } of checks)if (pattern.lastIndex = 0, patternMatchingCount.has(pattern) || patternMatchingCount.set(pattern, 0), pattern.test(util1)) for (let variant of (patternMatchingCount.set(pattern, patternMatchingCount.get(pattern) + 1), context.changedContent.push({
                    content: util1,
                    extension: "html"
                }), variants))context.changedContent.push({
                    content: variant + context.tailwindConfig.separator + util1,
                    extension: "html"
                });
                for (let [regex, count] of patternMatchingCount.entries())0 === count && _log.default.warn([
                    `The safelist pattern \`${regex}\` doesn't match any Tailwind CSS classes.`,
                    "Fix this pattern or remove it from your `safelist` configuration.",
                    "https://tailwindcss.com/docs/content-configuration#safelisting-classes"
                ]);
            }
        }
        let darkClassName = null !== (_concat_ = [].concat(null !== (_context_tailwindConfig_darkMode = context.tailwindConfig.darkMode) && void 0 !== _context_tailwindConfig_darkMode ? _context_tailwindConfig_darkMode : "media")[1]) && void 0 !== _concat_ ? _concat_ : "dark", parasiteUtilities = [
            prefix(context, darkClassName),
            prefix(context, "group"),
            prefix(context, "peer")
        ];
        context.getClassOrder = function(classes) {
            let sorted = [
                ...classes
            ].sort((a, z)=>a === z ? 0 : a < z ? -1 : 1), sortedClassNames = new Map(sorted.map((className)=>[
                    className,
                    null
                ])), rules = (0, _generateRules.generateRules)(new Set(sorted), context);
            rules = context.offsets.sort(rules);
            let idx = BigInt(parasiteUtilities.length);
            for (let [, rule] of rules)sortedClassNames.set(rule.raws.tailwind.candidate, idx++);
            return classes.map((className)=>{
                var _sortedClassNames_get;
                let order = null !== (_sortedClassNames_get = sortedClassNames.get(className)) && void 0 !== _sortedClassNames_get ? _sortedClassNames_get : null, parasiteIndex = parasiteUtilities.indexOf(className);
                return null === order && -1 !== parasiteIndex && (order = BigInt(parasiteIndex)), [
                    className,
                    order
                ];
            });
        }, context.getClassList = function(options = {}) {
            let output = [];
            for (let util of classList)if (Array.isArray(util)) {
                var _utilOptions_types, _utilOptions_modifiers, _context_tailwindConfig_theme_opacity, _utilOptions_values;
                let [utilName, utilOptions] = util, negativeClasses = [], modifiers = Object.keys(null !== (_utilOptions_modifiers = null == utilOptions ? void 0 : utilOptions.modifiers) && void 0 !== _utilOptions_modifiers ? _utilOptions_modifiers : {});
                (null == utilOptions ? void 0 : null === (_utilOptions_types = utilOptions.types) || void 0 === _utilOptions_types ? void 0 : _utilOptions_types.some(({ type  })=>"color" === type)) && modifiers.push(...Object.keys(null !== (_context_tailwindConfig_theme_opacity = context.tailwindConfig.theme.opacity) && void 0 !== _context_tailwindConfig_theme_opacity ? _context_tailwindConfig_theme_opacity : {}));
                let metadata = {
                    modifiers
                }, includeMetadata = options.includeMetadata && modifiers.length > 0;
                for (let [key, value] of Object.entries(null !== (_utilOptions_values = null == utilOptions ? void 0 : utilOptions.values) && void 0 !== _utilOptions_values ? _utilOptions_values : {})){
                    if (null == value) continue;
                    let cls = (0, _nameClass.formatClass)(utilName, key);
                    if (output.push(includeMetadata ? [
                        cls,
                        metadata
                    ] : cls), (null == utilOptions ? void 0 : utilOptions.supportsNegativeValues) && (0, _negateValue.default)(value)) {
                        let cls1 = (0, _nameClass.formatClass)(utilName, `-${key}`);
                        negativeClasses.push(includeMetadata ? [
                            cls1,
                            metadata
                        ] : cls1);
                    }
                }
                output.push(...negativeClasses);
            } else output.push(util);
            return output;
        }, context.getVariants = function() {
            let result = [];
            for (let [name, options] of context.variantOptions.entries()){
                var _options_values;
                options.variantInfo !== VARIANT_INFO.Base && result.push({
                    name,
                    isArbitrary: options.type === Symbol.for("MATCH_VARIANT"),
                    values: Object.keys(null !== (_options_values = options.values) && void 0 !== _options_values ? _options_values : {}),
                    hasDash: "@" !== name,
                    selectors ({ modifier , value  } = {}) {
                        var _context_variantMap_get, _options_values, _options_values_value, _options_values1;
                        let candidate = "__TAILWIND_PLACEHOLDER__", rule = _postcss.default.rule({
                            selector: `.${candidate}`
                        }), container = _postcss.default.root({
                            nodes: [
                                rule.clone()
                            ]
                        }), before = container.toString(), fns = (null !== (_context_variantMap_get = context.variantMap.get(name)) && void 0 !== _context_variantMap_get ? _context_variantMap_get : []).flatMap(([_, fn])=>fn), formatStrings = [];
                        for (let fn of fns){
                            let localFormatStrings = [], api = {
                                args: {
                                    modifier,
                                    value: null !== (_options_values_value = null === (_options_values = options.values) || void 0 === _options_values ? void 0 : _options_values[value]) && void 0 !== _options_values_value ? _options_values_value : value
                                },
                                separator: context.tailwindConfig.separator,
                                modifySelectors: (modifierFunction)=>(container.each((rule)=>{
                                        "rule" === rule.type && (rule.selectors = rule.selectors.map((selector)=>modifierFunction({
                                                get className () {
                                                    return (0, _generateRules.getClassNameFromSelector)(selector);
                                                },
                                                selector
                                            })));
                                    }), container),
                                format (str) {
                                    localFormatStrings.push(str);
                                },
                                wrap (wrapper) {
                                    localFormatStrings.push(`@${wrapper.name} ${wrapper.params} { & }`);
                                },
                                container
                            }, ruleWithVariant = fn(api);
                            if (localFormatStrings.length > 0 && formatStrings.push(localFormatStrings), Array.isArray(ruleWithVariant)) for (let variantFunction of ruleWithVariant)localFormatStrings = [], variantFunction(api), formatStrings.push(localFormatStrings);
                        }
                        let manualFormatStrings = [];
                        before !== container.toString() && (container.walkRules((rule)=>{
                            let modified = rule.selector, rebuiltBase = (0, _postcssSelectorParser.default)((selectors)=>{
                                selectors.walkClasses((classNode)=>{
                                    classNode.value = `${name}${context.tailwindConfig.separator}${classNode.value}`;
                                });
                            }).processSync(modified);
                            manualFormatStrings.push(modified.replace(rebuiltBase, "&").replace(candidate, "&"));
                        }), container.walkAtRules((atrule)=>{
                            manualFormatStrings.push(`@${atrule.name} (${atrule.params}) { & }`);
                        }));
                        let isArbitraryVariant = !(value in (null !== (_options_values1 = options.values) && void 0 !== _options_values1 ? _options_values1 : {}));
                        formatStrings = formatStrings.map((format)=>format.map((str)=>({
                                    format: str,
                                    isArbitraryVariant
                                }))), manualFormatStrings = manualFormatStrings.map((format)=>({
                                format,
                                isArbitraryVariant
                            }));
                        let opts = {
                            candidate,
                            context
                        }, result = formatStrings.map((formats)=>(0, _formatVariantSelector.finalizeSelector)(`.${candidate}`, (0, _formatVariantSelector.formatVariantSelector)(formats, opts), opts).replace(`.${candidate}`, "&").replace("{ & }", "").trim());
                        return manualFormatStrings.length > 0 && result.push((0, _formatVariantSelector.formatVariantSelector)(manualFormatStrings, opts).toString().replace(`.${candidate}`, "&")), result;
                    }
                });
            }
            return result;
        };
    }((corePluginList = Object.entries({
        ..._corePlugins.variantPlugins,
        ..._corePlugins.corePlugins
    }).map(([name, plugin])=>context.tailwindConfig.corePlugins.includes(name) ? plugin : null).filter(Boolean), userPlugins = context.tailwindConfig.plugins.map((plugin)=>(plugin.__isOptionsFunction && (plugin = plugin()), "function" == typeof plugin ? plugin : plugin.handler)), layerPlugins = [], root.each((node)=>{
        "atrule" === node.type && [
            "responsive",
            "variants"
        ].includes(node.name) && (node.name = "layer", node.params = "utilities");
    }), root.walkAtRules("layer", (layerRule)=>{
        if (function extractVariantAtRules(node) {
            node.walkAtRules((atRule)=>{
                [
                    "responsive",
                    "variants"
                ].includes(atRule.name) && (extractVariantAtRules(atRule), atRule.before(atRule.nodes), atRule.remove());
            });
        }(layerRule), "base" === layerRule.params) {
            for (let node of layerRule.nodes)layerPlugins.push(function({ addBase  }) {
                addBase(node, {
                    respectPrefix: !1
                });
            });
            layerRule.remove();
        } else if ("components" === layerRule.params) {
            for (let node1 of layerRule.nodes)layerPlugins.push(function({ addComponents  }) {
                addComponents(node1, {
                    respectPrefix: !1,
                    preserveSource: !0
                });
            });
            layerRule.remove();
        } else if ("utilities" === layerRule.params) {
            for (let node2 of layerRule.nodes)layerPlugins.push(function({ addUtilities  }) {
                addUtilities(node2, {
                    respectPrefix: !1,
                    preserveSource: !0
                });
            });
            layerRule.remove();
        }
    }), [
        ...corePluginList,
        _corePlugins.variantPlugins.pseudoElementVariants,
        _corePlugins.variantPlugins.pseudoClassVariants,
        _corePlugins.variantPlugins.ariaVariants,
        _corePlugins.variantPlugins.dataVariants,
        ...userPlugins,
        _corePlugins.variantPlugins.supportsVariants,
        _corePlugins.variantPlugins.directionVariants,
        _corePlugins.variantPlugins.reducedMotionVariants,
        _corePlugins.variantPlugins.prefersContrastVariants,
        _corePlugins.variantPlugins.darkVariants,
        _corePlugins.variantPlugins.printVariant,
        _corePlugins.variantPlugins.screenVariants,
        _corePlugins.variantPlugins.orientationVariants,
        ...layerPlugins
    ]), context), context;
}
let contextMap = _sharedState.contextMap, configContextMap = _sharedState.configContextMap, contextSourcesMap = _sharedState.contextSourcesMap;
function getContext(root, result, tailwindConfig, userConfigPath, tailwindConfigHash, contextDependencies) {
    let existingContext, sourcePath = result.opts.from;
    if (_sharedState.env.DEBUG && console.log("Source path:", sourcePath), null !== userConfigPath && contextMap.has(sourcePath)) existingContext = contextMap.get(sourcePath);
    else if (configContextMap.has(tailwindConfigHash)) {
        let context = configContextMap.get(tailwindConfigHash);
        contextSourcesMap.get(context).add(sourcePath), contextMap.set(sourcePath, context), existingContext = context;
    }
    let cssDidChange = (0, _cacheInvalidationJs.hasContentChanged)(sourcePath, root);
    if (existingContext) {
        let [contextDependenciesChanged, mtimesToCommit] = trackModified([
            ...contextDependencies
        ], getFileModifiedMap(existingContext));
        if (!contextDependenciesChanged && !cssDidChange) return [
            existingContext,
            !1,
            mtimesToCommit
        ];
    }
    if (contextMap.has(sourcePath)) {
        let oldContext = contextMap.get(sourcePath);
        if (contextSourcesMap.has(oldContext) && (contextSourcesMap.get(oldContext).delete(sourcePath), 0 === contextSourcesMap.get(oldContext).size)) {
            for (let [tailwindConfigHash1, context1] of (contextSourcesMap.delete(oldContext), configContextMap))context1 === oldContext && configContextMap.delete(tailwindConfigHash1);
            for (let disposable of oldContext.disposables.splice(0))disposable(oldContext);
        }
    }
    _sharedState.env.DEBUG && console.log("Setting up new context...");
    let context2 = createContext(tailwindConfig, [], root);
    Object.assign(context2, {
        userConfigPath
    });
    let [, mtimesToCommit1] = trackModified([
        ...contextDependencies
    ], getFileModifiedMap(context2));
    return configContextMap.set(tailwindConfigHash, context2), contextMap.set(sourcePath, context2), contextSourcesMap.has(context2) || contextSourcesMap.set(context2, new Set()), contextSourcesMap.get(context2).add(sourcePath), [
        context2,
        !0,
        mtimesToCommit1
    ];
}
