"use strict";
Object.defineProperty(exports, "__esModule", {
    value: !0
}), function(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: !0,
        get: all[name]
    });
}(exports, {
    normalize: ()=>normalize,
    url: ()=>url,
    number: ()=>number,
    percentage: ()=>percentage,
    length: ()=>length,
    lineWidth: ()=>lineWidth,
    shadow: ()=>shadow,
    color: ()=>color,
    image: ()=>image,
    gradient: ()=>gradient,
    position: ()=>position,
    familyName: ()=>familyName,
    genericName: ()=>genericName,
    absoluteSize: ()=>absoluteSize,
    relativeSize: ()=>relativeSize
});
const _color = require("./color"), _parseBoxShadowValue = require("./parseBoxShadowValue"), _splitAtTopLevelOnly = require("./splitAtTopLevelOnly");
let cssFunctions = [
    "min",
    "max",
    "clamp",
    "calc"
];
function isCSSFunction(value) {
    return cssFunctions.some((fn)=>RegExp(`^${fn}\\(.*\\)`).test(value));
}
const placeholder = "--tw-placeholder", placeholderRe = RegExp(placeholder, "g");
function normalize(value, isRoot = !0) {
    return value.startsWith("--") ? `var(${value})` : value.includes("url(") ? value.split(/(url\(.*?\))/g).filter(Boolean).map((part)=>/^url\(.*?\)$/.test(part) ? part : normalize(part, !1)).join("") : (value = value.replace(/([^\\])_+/g, (fullMatch, characterBefore)=>characterBefore + " ".repeat(fullMatch.length - 1)).replace(/^_/g, " ").replace(/\\_/g, "_"), isRoot && (value = value.trim()), value = value.replace(/(calc|min|max|clamp)\(.+\)/g, (match)=>{
        let vars = [];
        return match.replace(/var\((--.+?)[,)]/g, (match, g1)=>(vars.push(g1), match.replace(g1, placeholder))).replace(/(-?\d*\.?\d(?!\b-\d.+[,)](?![^+\-/*])\D)(?:%|[a-z]+)?|\))([+\-/*])/g, "$1 $2 ").replace(placeholderRe, ()=>vars.shift());
    }));
}
function url(value) {
    return value.startsWith("url(");
}
function number(value) {
    return !isNaN(Number(value)) || isCSSFunction(value);
}
function percentage(value) {
    return value.endsWith("%") && number(value.slice(0, -1)) || isCSSFunction(value);
}
function length(value) {
    return "0" === value || RegExp("^[+-]?[0-9]*\\.?[0-9]+(?:[eE][+-]?[0-9]+)?(?:cm|mm|Q|in|pc|pt|px|em|ex|ch|rem|lh|rlh|vw|vh|vmin|vmax|vb|vi|svw|svh|lvw|lvh|dvw|dvh|cqw|cqh|cqi|cqb|cqmin|cqmax)$").test(value) || isCSSFunction(value);
}
let lineWidths = new Set([
    "thin",
    "medium",
    "thick"
]);
function lineWidth(value) {
    return lineWidths.has(value);
}
function shadow(value) {
    for (let parsedShadow of (0, _parseBoxShadowValue.parseBoxShadowValue)(normalize(value)))if (!parsedShadow.valid) return !1;
    return !0;
}
function color(value) {
    let colors = 0;
    return !!(0, _splitAtTopLevelOnly.splitAtTopLevelOnly)(value, "_").every((part)=>!!(part = normalize(part)).startsWith("var(") || null !== (0, _color.parseColor)(part, {
            loose: !0
        }) && (colors++, !0)) && colors > 0;
}
function image(value) {
    let images = 0;
    return !!(0, _splitAtTopLevelOnly.splitAtTopLevelOnly)(value, ",").every((part)=>!!(part = normalize(part)).startsWith("var(") || !!(url(part) || gradient(part) || [
            "element(",
            "image(",
            "cross-fade(",
            "image-set("
        ].some((fn)=>part.startsWith(fn))) && (images++, !0)) && images > 0;
}
let gradientTypes = new Set([
    "linear-gradient",
    "radial-gradient",
    "repeating-linear-gradient",
    "repeating-radial-gradient",
    "conic-gradient"
]);
function gradient(value) {
    for (let type of (value = normalize(value), gradientTypes))if (value.startsWith(`${type}(`)) return !0;
    return !1;
}
let validPositions = new Set([
    "center",
    "top",
    "right",
    "bottom",
    "left"
]);
function position(value) {
    let positions = 0;
    return !!(0, _splitAtTopLevelOnly.splitAtTopLevelOnly)(value, "_").every((part)=>!!(part = normalize(part)).startsWith("var(") || !!(validPositions.has(part) || length(part) || percentage(part)) && (positions++, !0)) && positions > 0;
}
function familyName(value) {
    let fonts = 0;
    return !!(0, _splitAtTopLevelOnly.splitAtTopLevelOnly)(value, ",").every((part)=>!!(part = normalize(part)).startsWith("var(") || !(part.includes(" ") && !/(['"])([^"']+)\1/g.test(part) || /^\d/g.test(part)) && (fonts++, !0)) && fonts > 0;
}
let genericNames = new Set([
    "serif",
    "sans-serif",
    "monospace",
    "cursive",
    "fantasy",
    "system-ui",
    "ui-serif",
    "ui-sans-serif",
    "ui-monospace",
    "ui-rounded",
    "math",
    "emoji",
    "fangsong"
]);
function genericName(value) {
    return genericNames.has(value);
}
let absoluteSizes = new Set([
    "xx-small",
    "x-small",
    "small",
    "medium",
    "large",
    "x-large",
    "x-large",
    "xxx-large"
]);
function absoluteSize(value) {
    return absoluteSizes.has(value);
}
let relativeSizes = new Set([
    "larger",
    "smaller"
]);
function relativeSize(value) {
    return relativeSizes.has(value);
}
