"use strict";
function isScreenSortable(screen) {
    return 1 !== screen.values.length ? {
        result: !1,
        reason: "multiple-values"
    } : void 0 !== screen.values[0].raw ? {
        result: !1,
        reason: "raw-values"
    } : void 0 !== screen.values[0].min && void 0 !== screen.values[0].max ? {
        result: !1,
        reason: "min-and-max"
    } : {
        result: !0,
        reason: null
    };
}
function compareScreens(type, a, z) {
    let aScreen = toScreen(a, type), zScreen = toScreen(z, type), aSorting = isScreenSortable(aScreen), bSorting = isScreenSortable(zScreen);
    if ("multiple-values" === aSorting.reason || "multiple-values" === bSorting.reason) throw Error("Attempted to sort a screen with multiple values. This should never happen. Please open a bug report.");
    if ("raw-values" === aSorting.reason || "raw-values" === bSorting.reason) throw Error("Attempted to sort a screen with raw values. This should never happen. Please open a bug report.");
    if ("min-and-max" === aSorting.reason || "min-and-max" === bSorting.reason) throw Error("Attempted to sort a screen with both min and max values. This should never happen. Please open a bug report.");
    let { min: aMin , max: aMax  } = aScreen.values[0], { min: zMin , max: zMax  } = zScreen.values[0];
    a.not && ([aMin, aMax] = [
        aMax,
        aMin
    ]), z.not && ([zMin, zMax] = [
        zMax,
        zMin
    ]), aMin = void 0 === aMin ? aMin : parseFloat(aMin), aMax = void 0 === aMax ? aMax : parseFloat(aMax), zMin = void 0 === zMin ? zMin : parseFloat(zMin), zMax = void 0 === zMax ? zMax : parseFloat(zMax);
    let [aValue, zValue] = "min" === type ? [
        aMin,
        zMin
    ] : [
        zMax,
        aMax
    ];
    return aValue - zValue;
}
function toScreen(value, type) {
    return "object" == typeof value ? value : {
        name: "arbitrary-screen",
        values: [
            {
                [type]: value
            }
        ]
    };
}
function resolveValue({ "min-width": _minWidth , min =_minWidth , max , raw  } = {}) {
    return {
        min,
        max,
        raw
    };
}
Object.defineProperty(exports, "__esModule", {
    value: !0
}), function(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: !0,
        get: all[name]
    });
}(exports, {
    normalizeScreens: ()=>function normalizeScreens(screens, root = !0) {
            return Array.isArray(screens) ? screens.map((screen)=>{
                if (root && Array.isArray(screen)) throw Error("The tuple syntax is not supported for `screens`.");
                if ("string" == typeof screen) return {
                    name: screen.toString(),
                    not: !1,
                    values: [
                        {
                            min: screen,
                            max: void 0
                        }
                    ]
                };
                let [name, options] = screen;
                return (name = name.toString(), "string" == typeof options) ? {
                    name,
                    not: !1,
                    values: [
                        {
                            min: options,
                            max: void 0
                        }
                    ]
                } : Array.isArray(options) ? {
                    name,
                    not: !1,
                    values: options.map((option)=>resolveValue(option))
                } : {
                    name,
                    not: !1,
                    values: [
                        resolveValue(options)
                    ]
                };
            }) : normalizeScreens(Object.entries(null != screens ? screens : {}), !1);
        },
    isScreenSortable: ()=>isScreenSortable,
    compareScreens: ()=>compareScreens,
    toScreen: ()=>toScreen
});
