"use strict";
Object.defineProperty(exports, "__esModule", {
    value: !0
}), function(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: !0,
        get: all[name]
    });
}(exports, {
    updateAllClasses: ()=>updateAllClasses,
    asValue: ()=>asValue,
    parseColorFormat: ()=>parseColorFormat,
    asColor: ()=>asColor,
    asLookupValue: ()=>asLookupValue,
    typeMap: ()=>typeMap,
    coerceValue: ()=>coerceValue,
    getMatchingTypes: ()=>getMatchingTypes
});
const _escapeCommas = _interopRequireDefault(require("./escapeCommas")), _withAlphaVariable = require("./withAlphaVariable"), _dataTypes = require("./dataTypes"), _negateValue = _interopRequireDefault(require("./negateValue")), _validateFormalSyntax = require("./validateFormalSyntax"), _featureFlagsJs = require("../featureFlags.js");
function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    };
}
function updateAllClasses(selectors, updateClass) {
    selectors.walkClasses((sel)=>{
        sel.value = updateClass(sel.value), sel.raws && sel.raws.value && (sel.raws.value = (0, _escapeCommas.default)(sel.raws.value));
    });
}
function resolveArbitraryValue(modifier, validate) {
    if (!isArbitraryValue(modifier)) return;
    let value = modifier.slice(1, -1);
    if (validate(value)) return (0, _dataTypes.normalize)(value);
}
function asValue(modifier, options = {}, { validate =()=>!0  } = {}) {
    var _options_values;
    let value = null === (_options_values = options.values) || void 0 === _options_values ? void 0 : _options_values[modifier];
    return void 0 !== value ? value : options.supportsNegativeValues && modifier.startsWith("-") ? function(modifier, lookup = {}, validate) {
        let positiveValue = lookup[modifier];
        if (void 0 !== positiveValue) return (0, _negateValue.default)(positiveValue);
        if (isArbitraryValue(modifier)) {
            let resolved = resolveArbitraryValue(modifier, validate);
            if (void 0 === resolved) return;
            return (0, _negateValue.default)(resolved);
        }
    }(modifier.slice(1), options.values, validate) : resolveArbitraryValue(modifier, validate);
}
function isArbitraryValue(input) {
    return input.startsWith("[") && input.endsWith("]");
}
function splitUtilityModifier(modifier) {
    let slashIdx = modifier.lastIndexOf("/");
    return -1 === slashIdx || slashIdx === modifier.length - 1 || isArbitraryValue(modifier) && !modifier.includes("]/[") ? [
        modifier,
        void 0
    ] : [
        modifier.slice(0, slashIdx),
        modifier.slice(slashIdx + 1)
    ];
}
function parseColorFormat(value) {
    return "string" == typeof value && value.includes("<alpha-value>") ? ({ opacityValue =1  })=>value.replace("<alpha-value>", opacityValue) : value;
}
function unwrapArbitraryModifier(modifier) {
    return (modifier = modifier.slice(1, -1)).startsWith("--") && (modifier = `var(${modifier})`), modifier;
}
function asColor(modifier, options = {}, { tailwindConfig ={}  } = {}) {
    var _options_values, _options_values1, _options_values2, _tailwindConfig_theme, _tailwindConfig_theme_opacity, _options_values_color;
    if ((null === (_options_values = options.values) || void 0 === _options_values ? void 0 : _options_values[modifier]) !== void 0) return parseColorFormat(null === (_options_values1 = options.values) || void 0 === _options_values1 ? void 0 : _options_values1[modifier]);
    let [color, alpha] = splitUtilityModifier(modifier);
    if (void 0 !== alpha) {
        let normalizedColor = null !== (_options_values_color = null === (_options_values2 = options.values) || void 0 === _options_values2 ? void 0 : _options_values2[color]) && void 0 !== _options_values_color ? _options_values_color : isArbitraryValue(color) ? color.slice(1, -1) : void 0;
        if (void 0 === normalizedColor) return;
        if (normalizedColor = parseColorFormat(normalizedColor), isArbitraryValue(alpha)) return (0, _withAlphaVariable.withAlphaValue)(normalizedColor, unwrapArbitraryModifier(alpha));
        if ((null === (_tailwindConfig_theme = tailwindConfig.theme) || void 0 === _tailwindConfig_theme ? void 0 : null === (_tailwindConfig_theme_opacity = _tailwindConfig_theme.opacity) || void 0 === _tailwindConfig_theme_opacity ? void 0 : _tailwindConfig_theme_opacity[alpha]) === void 0) return;
        return (0, _withAlphaVariable.withAlphaValue)(normalizedColor, tailwindConfig.theme.opacity[alpha]);
    }
    return asValue(modifier, options, {
        validate: _dataTypes.color
    });
}
function asLookupValue(modifier, options = {}) {
    var _options_values;
    return null === (_options_values = options.values) || void 0 === _options_values ? void 0 : _options_values[modifier];
}
function guess(validate) {
    return (modifier, options)=>asValue(modifier, options, {
            validate
        });
}
let typeMap = {
    any: asValue,
    color: asColor,
    url: guess(_dataTypes.url),
    image: guess(_dataTypes.image),
    length: guess(_dataTypes.length),
    percentage: guess(_dataTypes.percentage),
    position: guess(_dataTypes.position),
    lookup: asLookupValue,
    "generic-name": guess(_dataTypes.genericName),
    "family-name": guess(_dataTypes.familyName),
    number: guess(_dataTypes.number),
    "line-width": guess(_dataTypes.lineWidth),
    "absolute-size": guess(_dataTypes.absoluteSize),
    "relative-size": guess(_dataTypes.relativeSize),
    shadow: guess(_dataTypes.shadow),
    size: guess(_validateFormalSyntax.backgroundSize)
}, supportedTypes = Object.keys(typeMap);
function coerceValue(types, modifier, options, tailwindConfig) {
    if (options.values && modifier in options.values) for (let { type  } of null != types ? types : []){
        let result = typeMap[type](modifier, options, {
            tailwindConfig
        });
        if (void 0 !== result) return [
            result,
            type,
            null
        ];
    }
    if (isArbitraryValue(modifier)) {
        let idx, arbitraryValue = modifier.slice(1, -1), [explicitType, value] = -1 === (idx = arbitraryValue.indexOf(":")) ? [
            void 0,
            arbitraryValue
        ] : [
            arbitraryValue.slice(0, idx),
            arbitraryValue.slice(idx + 1)
        ];
        if (/^[\w-_]+$/g.test(explicitType)) {
            if (void 0 !== explicitType && !supportedTypes.includes(explicitType)) return [];
        } else value = arbitraryValue;
        if (value.length > 0 && supportedTypes.includes(explicitType)) return [
            asValue(`[${value}]`, options),
            explicitType,
            null
        ];
    }
    for (let match of getMatchingTypes(types, modifier, options, tailwindConfig))return match;
    return [];
}
function* getMatchingTypes(types, rawModifier, options, tailwindConfig) {
    let modifiersEnabled = (0, _featureFlagsJs.flagEnabled)(tailwindConfig, "generalizedModifiers"), [modifier, utilityModifier] = splitUtilityModifier(rawModifier);
    if (modifiersEnabled && null != options.modifiers && ("any" === options.modifiers || "object" == typeof options.modifiers && (utilityModifier && isArbitraryValue(utilityModifier) || utilityModifier in options.modifiers)) || (modifier = rawModifier, utilityModifier = void 0), void 0 !== utilityModifier && "" === modifier && (modifier = "DEFAULT"), void 0 !== utilityModifier && "object" == typeof options.modifiers) {
        var _options_modifiers, _options_modifiers_utilityModifier;
        let configValue = null !== (_options_modifiers_utilityModifier = null === (_options_modifiers = options.modifiers) || void 0 === _options_modifiers ? void 0 : _options_modifiers[utilityModifier]) && void 0 !== _options_modifiers_utilityModifier ? _options_modifiers_utilityModifier : null;
        null !== configValue ? utilityModifier = configValue : isArbitraryValue(utilityModifier) && (utilityModifier = unwrapArbitraryModifier(utilityModifier));
    }
    for (let { type  } of null != types ? types : []){
        let result = typeMap[type](modifier, options, {
            tailwindConfig
        });
        void 0 !== result && (yield [
            result,
            type,
            null != utilityModifier ? utilityModifier : null
        ]);
    }
}
